-------------------------------------------------------------------------------
-- Addon - Combo-Points - Init
-------------------------------------------------------------------------------
-- Version
local MAJOR = "Avion2_ComboPoints"
local MINOR = tonumber(("2"):match("%d+")) or 0

-- Dependencies
if not Avion2Core 				then error(MAJOR .. " requires Avion2.") end
if not LibStub("AceEvent-3.0")	then error(MAJOR .. " requires AceEvent-3.0.") end

-- Create
Avion2_ComboPoints 			= Avion2Core:NewModule(MAJOR, "AceEvent-3.0")
Avion2_ComboPoints.major 	= MAJOR
Avion2_ComboPoints.minor 	= MINOR


-------------------------------------------------------------------------------
-- Addon - Combo-Points - Variables
-------------------------------------------------------------------------------
-- Translation
local L 	= LibStub("AceLocale-3.0"):GetLocale(MAJOR)
local coreL	= LibStub("AceLocale-3.0"):GetLocale(Avion2Core.major)

-- Local functions
local ComboPointUpdate


-------------------------------------------------------------------------------
-- Addon - Combo-Points - Register with core
-------------------------------------------------------------------------------
-- Register SavedVariables
function Avion2_ComboPoints:RegisterModuleDB()
	-- Register own saved variables DB
	self.db = Avion2Core.db:RegisterNamespace(Avion2_ComboPoints.major, {
		profile = {
			comboPoints = {
				[0] = {
					animationList 	= { },
					cancelAnimation	= 0,
					enabled 		= true,
				},
				[1] = {
					animationList 	= { },
					cancelAnimation	= 0,
					enabled 		= true,
				},
				[2] = {
					animationList 	= { },
					cancelAnimation	= 0,
					enabled 		= true,
				},
				[3] = {
					animationList 	= { },
					cancelAnimation	= 0,
					enabled 		= true,
				},
				[4] = {
					animationList 	= { },
					cancelAnimation	= 0,
					enabled 		= true,
				},
				[5] = {
					animationList 	= { },
					cancelAnimation	= 0,
					enabled 		= true,
				},
			},
			cancelOnlyLesser	= true,
			lastMinor 			= MINOR,
		},
	})
	
	-- Last combo-points
	self.lastComboPoints = 0
	
	-- Updateto to new rev, data converters should go here
	self.db.profile.lastMinor = Avion2_ComboPoints.minor
end


-- (Un)register (un)needed events
function Avion2_ComboPoints:UpdateEventRegistration()
	local enabled = false
	
	-- Scan if combo point is enabled
	if (self:IsEnabled()) then
		for _, data in pairs(self.db.profile.comboPoints) do
			if (data.enabled) then
				enabled = true
				break
			end
		end
	end
		
	-- Combo-Points events
	if (enabled) then
		self:RegisterEvent("PLAYER_COMBO_POINTS", ComboPointUpdate)
	else
		self:UnregisterEvent("PLAYER_COMBO_POINTS")
	end
end


-- Update animation (convert animationFunc: string name to function pointer)
function Avion2_ComboPoints:LoadAnimationTables()
	for _, data in pairs(self.db.profile.comboPoints) do
		Avion2Core:LoadAnimationTables(data.animationList)
	end
end


-------------------------------------------------------------------------------
-- Addon - Combo-Points - Options
-------------------------------------------------------------------------------
-- Fetch animationTable from animationID (as suplied by eg. Menu_Reuqest)
function Avion2_ComboPoints:GetAnimationTable(animationID)
	success, _, arg1, arg2 = self:Deserialize(animationID)
	
	if (success) then
		return self.db.profile.comboPoints[arg1].animationList[arg2]
	else
		return false
	end
end


-- Build Optionstable
function Avion2_ComboPoints:UpdateOptionTable(args)
	-- Update options when enabled and initialized
	if (self.db and self:IsEnabled()) then
		local serializedInfo
		
		-- Final options
		args[self.major] = {
			type = 'group',
			name = L["ComboPoints"],
			desc = L["ComboPoints_Desc"],
			args = {
				cancelOnlyLesser = {
					order = 1,
					type = 'toggle',
					name = L["CancelOnlyLesser"],
					desc = L["CancelOnlyLesser_Desc"],
					get = function()
						return self.db.profile.cancelOnlyLesser
					end,
					set = function()
						self.db.profile.cancelOnlyLesser = not self.db.profile.cancelOnlyLesser
					end,
				},
			},
		}
		
		-- Options for each compo-point
		for index, data in pairs(self.db.profile.comboPoints) do
			args[self.major].args[string.format("ComboPoint_%d", index)] = {
				type = 'group',
				name = string.format(L["ComboPoint"], index),
				desc = string.format(L["ComboPoint_Desc"], index),
				args = { 
					list = {
						order = 1,
						type = 'group',
						name = coreL["AnimationList"],
						desc = coreL["AnimationList_Desc"],
						args = {
							new = Avion2Core:GetDefaultNewAnimationOption(data.animationList),
						},
					},
					cancelAnimation = {
						order = 2,
						type = 'select',
						name = coreL["CancelAnimation"],
						desc = coreL["CancelAnimation_Desc"],
						values = {
							[0] = coreL["DontCancel"],
							[1] = coreL["FadeOut"],
							[2] = coreL["Cancel"],
						},
						get = function()
							return data.cancelAnimation
						end,
						set = function(info, val)
							data.cancelAnimation = val
						end,
					},
					enabled = {
						order = 3,
						type = 'toggle',
						name = coreL["Enabled"],
						desc = coreL["Enabled_Desc"],
						get = function()
							return data.enabled
						end,
						set = function()
							data.enabled = not data.enabled
							
							self:UpdateEventRegistration()
						end,
					},
				},
			}
			-- Add options for each animation
			for animationListID, _ in pairs(data.animationList) do
				-- Combine all info to find this animation in module db in ONE string
				serializedInfo = self:Serialize(Avion2_ComboPoints.major, index, animationListID)
				
				-- Update option table
				args[self.major].args[string.format("ComboPoint_%d", index)].args.list.args[string.format("Animation_%d", animationListID)] = Avion2Core:GetDefaultAnimationOption(data.animationList, animationListID, serializedInfo)
			end
		end

	-- Disabled, remove options
	else
		args[self.major] = nil
	end
end


-------------------------------------------------------------------------------
-- Addon - Combo-Points - Events
-------------------------------------------------------------------------------
-- Combo-points changed
ComboPointUpdate = function()
	local comboPoints = GetComboPoints()
	
	-- Test if there is a trigger for this ...
	if (Avion2_ComboPoints.db.profile.comboPoints[comboPoints] and Avion2_ComboPoints.db.profile.comboPoints[comboPoints].enabled) then
		Avion2Core:TriggerAnimationList(string.format("Avion2_ComboPoints_%d", comboPoints), Avion2_ComboPoints.db.profile.comboPoints[comboPoints].animationList)
	end
	
	-- Cancel only if cancelOnlyLesser is false or (cancelOnlyLesser is true and) now less cp than before
	if (not (Avion2_ComboPoints.db.profile.cancelOnlyLesser and self.lastComboPoints > comboPoints)) then
		-- Cancel animations from triggers (combo-points) that aren't active aniymore
		for index, data in pairs(self.db.profile.comboPoints) do
			if (data.cancelAnimation ~= 0) then
				Avion2Core:TriggerAnimationList(string.format("Avion2_ComboPoints_%d", index), data.animationList, (data.cancelAnimation == 1))
			end
		end
	end
	
	self.lastComboPoints = comboPoints
end
